import React from 'react'
import ReactDOM from 'react-dom'
import PropChangeCounter from './propchangecounter.jsx'

export class PageApp extends React.Component 
{
  // uncomment to see what happens if you don't use the .bind() method, line [line 38]
  //randomPieceOfData = "some random piece of data";

  anotherHandler = (tagName) =>{
    return (event) =>{
      this.handleChange(event.target.value);
    }
  }
  constructor(props) {
    super(props)
    this.state = { email: '' }
  }

  // In the first <input> example we attach to the onChange property an arrow head function
  // this takes on the 'this object' of the object in scope

  // In the second <input> example we attach an inline arrow head function, this is
  // instantiated with each onChange event

  // In the first <PropChangeCounter> example onChange is just a property, we need
  // to write our own mechanism to trigger a call to the callback, see propchangecounter.jsx
  // The callback here is an arrow head function, so it is instatiated each time
  // the <PropChangeCounter> is rendered.  The arrow head function can directly call
  // the method this.handleChange() because it takes on the scope of the current
  // 'this object'
  
  // In the second <PropChangeCounter> example onChange is just a property, we need
  // to write our own mechanism to trigger a call to the callback, see propchangecounter.jsx
  // The callback here is a method in the scope of the current 'this object' and 
  // it is NOT instatiated each time the <PropChangeCounter> is rendered.  
  // The 'this.handleChange' does not bind the method to the current 'this object' 
  // in scope, so we must use 'this.handleChange.bind(this)' to attach the current
  // 'this object' to handleChange() when it is called
  
  render() {
    return <div>
      <input placeholder="Email"
        type="text" 
        name="some_name"
        onChange={this.anotherHandler("some_name")}
      />
      <input
        placeholder="Email"
        value={this.state.email}
        onChange={e => {
          console.log( "input.onChange()");
          this.setState({ email: e.target.value })}
        }
      />
      <PropChangeCounter
        constant={"1:this doesn't change"}
        value={this.state.email}
        onChange={c =>
          {
            console.log( "PropChangeCounter.onChange()");
            this.handleChange(c)
          }
        }
      />
      <PropChangeCounter
        constant={"2:this doesn't change"}
        value={this.state.email}
        onChange={this.handleChange.bind(this)}
      />
    </div>
  }

  handleChange( ct )
  {
    console.log( "handleChange(): " + ct );

    // uncomment to see what happens if the .bind() [line 38] isn't used
    //console.log( this.randomPieceOfData );
  }
}


